@@ -464,9 +464,12 @@ class BaseProject:
             if self.dims == 1: # 1D
                 fig = plt.figure()
                 points_dict = {}
-                for label_idx in range(self.n_classes):
-                    points_dict.update({label_idx: self.X[self.y==label_idx]})
-                    plt.scatter(x=points_dict[label_idx][:,0], y=points_dict[label_idx][:,1], label=label_idx)
+                if self.n_classes:
+                    for label_idx in range(self.n_classes):
+                        points_dict.update({label_idx: self.X[self.y==label_idx]})
+                        plt.scatter(x=points_dict[label_idx][:,0], y=points_dict[label_idx][:,1], label=label_idx)
+                else:
+                    plt.scatter(x=self.X[:,0], y=self.X[:,1])
                 plt.title('Data Distribution')
                 plt.xlabel('positions')
                 plt.legend(loc=(1.05,1.05))
@@ -476,9 +479,12 @@ class BaseProject:
             elif self.dims == 2 : # 2D
                 fig = plt.figure()
                 points_dict = {}
-                for label_idx in range(self.n_classes):
-                    points_dict.update({label_idx: self.X[self.y==label_idx]})
-                    plt.scatter(x=points_dict[label_idx][:,0], y=points_dict[label_idx][:,1], label=label_idx)
+                if self.n_classes:
+                    for label_idx in range(self.n_classes):
+                        points_dict.update({label_idx: self.X[self.y==label_idx]})
+                        plt.scatter(x=points_dict[label_idx][:,0], y=points_dict[label_idx][:,1], label=label_idx)
+                else: #regression
+                    plt.scatter(x=self.X[:,0], y=self.X[:,1])
                 plt.title('Data Distribution')
                 plt.xlabel('x positions')
                 plt.ylabel('y positions')
@@ -490,9 +496,12 @@ class BaseProject:
                 fig1 = plt.figure()
                 ax = fig1.add_subplot(111, projection='3d')
                 points_dict = {}
-                for label_idx in range(self.n_classes):
-                    points_dict.update({label_idx: self.X[self.y==label_idx]})
-                    ax.scatter(xs=points_dict[label_idx][:,0], ys=points_dict[label_idx][:,1], zs= points_dict[label_idx][:,2], label=label_idx)
+                if self.n_classes:
+                    for label_idx in range(self.n_classes):
+                        points_dict.update({label_idx: self.X[self.y==label_idx]})
+                        ax.scatter(xs=points_dict[label_idx][:,0], ys=points_dict[label_idx][:,1], zs= points_dict[label_idx][:,2], label=label_idx)
+                else:
+                    ax.scatter(xs=self.X[:,0], ys=self.X[:,1], zs= self.X[:,2])
                 ax.set_xlabel('x positions')
                 ax.set_ylabel('y positions')
                 ax.set_zlabel('z positions')
@@ -505,9 +514,12 @@ class BaseProject:
                 fig2 = plt.figure()
                 ax = fig2.add_subplot(111, projection='3d')
                 points_dict = {}
-                for label_idx in range(self.n_classes):
-                    points_dict.update({label_idx: self.X[self.y==label_idx]})
-                    ax.scatter(xs=points_dict[label_idx][:,0], ys=points_dict[label_idx][:,1], zs= points_dict[label_idx][:,2], label=label_idx)
+                if self.n_classes:
+                    for label_idx in range(self.n_classes):
+                        points_dict.update({label_idx: self.X[self.y==label_idx]})
+                        ax.scatter(xs=points_dict[label_idx][:,0], ys=points_dict[label_idx][:,1], zs= points_dict[label_idx][:,2], label=label_idx)
+                else:
+                    ax.scatter(xs=self.X[:,0], ys=self.X[:,1], zs= self.X[:,2])
                 ax.set_xlabel('x positions')
                 ax.set_ylabel('y positions')
                 ax.set_zlabel('z positions')
@@ -521,9 +533,12 @@ class BaseProject:
                 fig3 = plt.figure()
                 ax = fig3.add_subplot(111, projection='3d')
                 points_dict = {}
-                for label_idx in range(self.n_classes):
-                    points_dict.update({label_idx: self.X[self.y==label_idx]})
-                    ax.scatter(xs=points_dict[label_idx][:,0], ys=points_dict[label_idx][:,1], zs= points_dict[label_idx][:,2], label=label_idx)
+                if self.n_classes:
+                    for label_idx in range(self.n_classes):
+                        points_dict.update({label_idx: self.X[self.y==label_idx]})
+                        ax.scatter(xs=points_dict[label_idx][:,0], ys=points_dict[label_idx][:,1], zs=points_dict[label_idx][:,2], label=label_idx)
+                else:
+                    ax.scatter(xs=self.X[:,0], ys=self.X[:,1], zs= self.X[:,2])
                 ax.set_xlabel('x positions')
                 ax.set_ylabel('y positions')
                 ax.set_zlabel('z positions')
@@ -537,9 +552,12 @@ class BaseProject:
                 fig4 = plt.figure()
                 ax = fig4.add_subplot(111, projection='3d')
                 points_dict = {}
-                for label_idx in range(self.n_classes):
-                    points_dict.update({label_idx: self.X[self.y==label_idx]})
-                    ax.scatter(xs=points_dict[label_idx][:,0], ys=points_dict[label_idx][:,1], zs= points_dict[label_idx][:,2], label=label_idx)
+                if self.n_classes:
+                    for label_idx in range(self.n_classes):
+                        points_dict.update({label_idx: self.X[self.y==label_idx]})
+                        ax.scatter(xs=points_dict[label_idx][:,0], ys=points_dict[label_idx][:,1], zs= points_dict[label_idx][:,2], label=label_idx)
+                else:
+                    ax.scatter(xs=self.X[:,0], ys=self.X[:,1], zs=self.X[:,2])
                 ax.set_xlabel('x positions')
                 ax.set_ylabel('y positions')
                 ax.set_zlabel('z positions')
@@ -557,9 +575,12 @@ class BaseProject:
                 pca_data = pca.fit_transform(self.X)
                 fig = plt.figure()
                 points_dict = {}
-                for label_idx in range(self.n_classes):
-                    points_dict.update({label_idx: pca_data[self.y==label_idx]})
-                    plt.scatter(x=points_dict[label_idx][:,0], y=points_dict[label_idx][:,1], label=label_idx)
+                if self.n_classes:
+                    for label_idx in range(self.n_classes):
+                        points_dict.update({label_idx: pca_data[self.y==label_idx]})
+                        plt.scatter(x=points_dict[label_idx][:,0], y=points_dict[label_idx][:,1], label=label_idx)
+                else:
+                    plt.scatter(x=pca_data[:,0], y=pca_data[:,1])
                 plt.title('Data Distribution')
                 plt.xlabel('1st principal axis')
                 plt.ylabel('2nd principal axis')
@@ -791,14 +812,14 @@ class BaseProject:
         mode = 'min' if val_metric == 'val/loss' else 'max'
         random_state_dir_list = self.get_available_random_states()
         for random_state_dir in random_state_dir_list:
-            jsonlog_dir = random_state_dir+"/logs/jsonlogs"
+            jsonlog_dir = f"{random_state_dir}/logs/jsonlogs"
             for model_type_dir in os.listdir(jsonlog_dir):
-                for metadata_dir in os.listdir(jsonlog_dir + "/" + model_type_dir):
+                for metadata_dir in os.listdir(f"{jsonlog_dir}/{model_type_dir}"):
                     if str(metadata_dir) == metadata: 
                         best_val = None
-                        for version in os.listdir(jsonlog_dir + "/" + model_type_dir + "/" + metadata_dir):
-                            for file in os.listdir(jsonlog_dir + "/" + model_type_dir + "/" + metadata_dir + "/" + version):
-                                file_path = jsonlog_dir + "/" + model_type_dir + "/" + metadata_dir + "/" + version + "/" + file
+                        for version in os.listdir(f"{jsonlog_dir}/{model_type_dir}/{metadata_dir}"):
+                            for file in os.listdir(f"{jsonlog_dir}/{model_type_dir}/{metadata_dir}/{version}"):
+                                file_path = f"{jsonlog_dir}/{model_type_dir}/{metadata_dir}/{version}/{file}"
                                 with open(file_path, "r") as json_file:
                                     logs = json.load(json_file)
                                     val_score = logs[val_metric] if isinstance(logs[val_metric], float) else logs[val_metric][-1]
